# -*- coding: utf8 -*-

from datetime import datetime, timedelta
from time import sleep

import httplib2
import json
import logging
import logger
import sys
import time
from thrift.transport import TTransport
from thrift.transport.TSocket import TSocket
from thrift.protocol import TBinaryProtocol

import ConfigParser
from serviceLock import ServiceLock
from syncThrift import CashCommunicationService

log = logging.getLogger('su.artix.loggerSync')


def checkUnloadRequired(host, port, cashCode):
    """
    Функция получения даты и времени последней выгрузки продаж
    :param host - порт REST сервера
    :param port - порт REST сервера
    :param cashCode - код кассы
    """
    log.info(u"Получение времени последней выгрузки продаж")
    http = httplib2.Http()
    requestUrl = "http://%s:%s/CSrest/rest/cashes/%s/state" % (host, port, cashCode)
    log.info(requestUrl)
    response, content = http.request(requestUrl)
    log.debug(content)
    jsonResponse = json.loads(content)
    errorMessage = u"Ошибка при получении времени последней продажи с сервера"

    if response.status == 200 and "lastSaleTime" in jsonResponse:
        return True, jsonResponse["lastSaleTime"]
    elif response.status == 200 and len(jsonResponse) and "message" not in jsonResponse and "errorCode" not in jsonResponse:
        fromDate = datetime.now() - timedelta(weeks=4)
        fromDateStr = datetime.strftime(fromDate, "%Y-%m-%dT%H:%M:%S.000+0000")

        log.warn(u"Не удалось получить время последней продажи. Будет выполнена выгрузка продаж за последние 4 недели")
        return True, fromDateStr
    elif response.status == 400 and "message" in jsonResponse and "errorCode" in jsonResponse:
        errorMessage = u"%s: %s" % (errorMessage, jsonResponse["message"])

    raise Exception(errorMessage)

def getClient():
    """
    Функция создает клиента для подключение к Thrift-серверу SyncServer
    """
    socket = TSocket()
    socket.port = int(7795)
    socket.setTimeout(20000)
    transport = TTransport.TBufferedTransport(socket)
    transport.open()
    return CashCommunicationService.Client(TBinaryProtocol.TBinaryProtocol(transport))


def unloadSales(fromDate):
    """
    Функция подключение к SyncServer и выполнение функции выгрузки продаж
    Период выгрузки с @param по текущее время
    """
    log.debug(u"Принудительная выгрузка продаж")
    client = getClient()

    timeWithoutTimeZone = datetime.strptime(fromDate[:-5], '%Y-%m-%dT%H:%M:%S.%f')
    fromDate = timeWithoutTimeZone - timedelta(seconds=time.timezone)
    fromDateStr = datetime.strftime(fromDate, "%Y-%m-%dT%H:%M:%S.0")
    toDate = datetime.now()
    toDateStr = datetime.strftime(toDate, "%Y-%m-%dT%H:%M:%S.0")

    log.info(u"Принудительная выгрузка продаж за период с %s по %s" % (fromDateStr, toDateStr))
    client.unload(str(), fromDateStr, toDateStr, None, None, None)

if __name__ == "__main__":

    serviceLock = ServiceLock('/linuxcash/cash/sync/bin/watchdog-unload-pidfile', log)

    if serviceLock.lock(True):
        log.info(u'Запуск принудительной выгрузки продаж')
    else:
        exit(0)

    try:
        reconnectCount = int(sys.argv[1])
        if reconnectCount > 0:
            reconnectPeriod = int(sys.argv[2])
        elif reconnectCount < 0:
            reconnectCount = 0
    except Exception, e:
        reconnectPeriod = 20
        reconnectCount = 5
        log.warn(u"Некорректные параметры. Используются параметры по умолчанию (%s попыток с периодом %s секунд)" %
                 (reconnectCount, reconnectPeriod))
        log.warn(unicode(e))

    try:
        cp = ConfigParser.ConfigParser()
        cp.read('../properties/cash.ini')
        host = cp.get('Properties', 'dictUrlHostCS')
        cashCode = cp.get('Properties', 'cash.code')
        port = cp.get('Properties', 'cashServerRestPort')
    except Exception, e:
        log.warn(u"Не удалось получить значения параметров сервера или код кассы, завершение работы")
        log.warn(unicode(e))
        sys.exit(0)

    result = False

    for i in range(0, reconnectCount + 1):
        try:
            unloadRequired, dateFrom = checkUnloadRequired(host, port, cashCode)
            if unloadRequired:
                unloadSales(dateFrom)
            result = True
            break
        except Exception, e:
            log.error(u"В процессе выполнения операции произошла ошибка")
            log.error(unicode(e))

        if reconnectCount-i == 0:
            break

        log.warn(u"Повторная попытка через %s секунд. Осталось попыток: %s" % (reconnectPeriod, reconnectCount-i))
        sleep(reconnectPeriod)

    if result:
        log.info(u"Операция выполнена")
    else:
        log.warn(u"Не удалось выполнить принудительную выгрузку продаж")

    serviceLock.unlock()
