# -*- coding: UTF-8 -*-

import sys, os, errno
import logging


class ServiceLock(object):
    """
    Объект отвечающий за файл блокировки
    В конструктор передается имя файла блокировки
    """
    def __init__(self, filename, logger):
        self.lockFileName = filename
        self.log = logger

    def lock(self, exitOnLock = True):
        """
        Создает файл блокировки. Если файл уже есть и процесс с этим pid'ом запущен,
        завершаем работу.
        @param exitOnLock если true и сервис уже запущен, выполнять выход
        @return true если блокировка выполнена
        """
        if os.path.exists(self.lockFileName):
            self.log.info(u'Найден файл блокировки %s' % self.lockFileName)
            file = open(self.lockFileName, 'r')
            pid = int(file.read())
            file.close()
            if self.pid_exists(pid):
                self.log.error(u'Сервис уже запущен. pid = %s' % pid)
                if exitOnLock:
                    os._exit(1) # выход без выполнения atexit
                return False
            else:
                self.log.warn(u'Предыдущая сессия была завершена с ошибкой')
        if not os.path.exists(os.path.dirname(self.lockFileName)):
            os.makedirs(os.path.dirname(self.lockFileName))
        self.log.info(u'Создание файла блокировки %s' % self.lockFileName)
        file = open(self.lockFileName, 'w')
        file.write(str(os.getpid()))
        file.close()
        return True

    def unlock(self):
        """
        Удаляет файл блокировки
        """
        if os.path.exists(self.lockFileName):
            self.log.info(u'Удаление файла блокировки %s' % self.lockFileName)
            os.remove(self.lockFileName)

    def pid_exists(self, pid):
        """
        Возвращает true если процесс с указанным pid запущен
        """
        try:
            os.kill(pid, 0)
        except OSError as e:
            return e.errno == errno.EPERM
        else:
            return True